#!/usr/bin/perl -w -I/usr/sausalito/perl

use strict;

use CCE;
use Sauce::Util;
use Sauce::Validators;

#declares that should probably go elsewhere:
my $service         = "cobalt-networker";
my $scriptFile      = "/etc/rc.d/init.d/cobalt-networker";
my $chkconf         = "/sbin/chkconfig";
my $confFile	     = "/nsr/res/servers";
my $confBlockStart = "# Legato Networker Config Start"; # Comment block for config files
my $confBlockStop = "# Legato Networker Config Stop";
my $port_cmd = '/usr/bin/nsrports';

my $cce = new CCE ( Namespace => "Networker" );

$cce->connectfd();

# Need to test if event_object successful... what do we look at?
my $networker_obj = $cce->event_object();
my @lgNetworkerHost = $cce->scalar_to_array($networker_obj->{lgNetworkerHost});

my $enabled = $networker_obj->{enable};

if ($enabled && !@lgNetworkerHost) {
    $cce->warn("[[legato-networker.enabledButNoData]]");
    $cce->bye("FAIL");
    die;
}

my $hostname = `hostname`;
chomp($hostname);
if ($enabled && !checkDNSLookup($hostname)) {
    $cce->warn("[[legato-networker.lgNetworkerLocalHost_LookupFailed]]");
    $cce->bye("FAIL");
    die;
}

foreach my $server (@lgNetworkerHost) {
    if ($enabled && !checkDNSLookup($server)) {
	$cce->warn("[[legato-networker.lgNetworkerHostField_LookupFailed]]");
	$cce->bye("FAIL");
	die;
    }
    if ($enabled) {
	my $ret=`/usr/sbin/nsradmin -i /nsr/res/cmd -s $server 2>&1`;
	if($ret=~/RPC error/){
		$cce->warn("[[legato-networker.badHostname]]");
		$cce->bye("FAIL");
		die;
	}
    }
}

# Add hostnames to /nsr/res/servers
my $data=join "\n",@lgNetworkerHost;

if(! -e $confFile){
	system("/bin/mkdir",'-p',"/nsr/res");
}

Sauce::Util::replaceblock($confFile, $confBlockStart, $data,$confBlockStop,0600);

# Start service

if ($networker_obj->{enable}) {
	system ($chkconf,'--add',$service) &&
		($cce->warn("[[legato-networker.cannotCreateSymlink]] $!") &&
		 $cce->bye("FAIL") &&
		 die);

	system ("$scriptFile start > /dev/null 2>&1") &&
		($cce->warn("[[legato-networker.cannotStartLegatoServer]]: $!") &&
		$cce->bye("FAIL") &&
		 die);
	
	# see if the ports need to be massaged
	my $new_ports = $cce->event_new();

	if ($new_ports->{enable} || $new_ports->{servicePortLow} 
		|| $new_ports->{servicePortHigh}) {
		
		if (system ($port_cmd, '-S', 
			$networker_obj->{servicePortLow} . 
			'-' . $networker_obj->{servicePortHigh})) {
			
			$cce->warn('[[legato-networker.cantChangeServPorts]]');
			$cce->bye('FAIL');
			exit 1;
		}
	}

	if ($new_ports->{enable} || $new_ports->{connPortLow} 
		|| $new_ports->{connPortHigh}) {
		
		if (system ($port_cmd, '-C',
                        $networker_obj->{connPortLow} . 
                        '-' . $networker_obj->{connPortHigh})) {

                        $cce->warn('[[legato-networker.cantChangeConnPorts]]');
                        $cce->bye('FAIL');
                        exit 1;
                }
        }

 }else{
	system($chkconf,'--del',$service) &&
		($cce->warn("[[legato-networker.cannotBreakLink]]: $!") &&
		$cce->bye("FAIL"));

	system ("$scriptFile stop > /dev/null 2>&1") &&
		($cce->warn("[[legato-networker.cannotStopLegatoServer]]: $!") &&
		$cce->bye("FAIL")); 
}
	
$cce->bye("SUCCESS");

# The basic algorithm is this:
# hostname => ip addresses
# for each address, ipAddress => hostname2 => ipAddress2
# make sure ipAddress == ipAddress2. If they are all equal, good. If not, bad.
# Here are 2 examples:
# --------------------
# www.yahoo.com => 204.71.202.160, 216.115.105.2, 204.71.200.74,
#                  204.71.200.75, 204.71.200.67, 204.71.200.68
# 204.71.202.160 => w1.yahoo.com => 204.71.202.160
#
# They match. Good.
# --------------------
# yahoo.com => 204.71.200.243, 204.71.200.245
# 204.71.200.243 => UNKNOWN.yahoo.com => does not exist
# 204.71.200.245 => UNKNOWN.yahoo.com => does not exist
# They don't exist. Bad
sub checkDNSLookup {
    use Socket;
    my $host = shift; 
    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname($host) or return 0;
    my $ipaddr;
    foreach $ipaddr (@addrs) {
	my $revHost = gethostbyaddr($ipaddr, AF_INET) or return 0;
	my $revHostToIP =  gethostbyname($revHost) or return 0;
	if ($ipaddr != $revHostToIP) {
	    return 0;
	}
    }
    return 1;
}
